      SUBROUTINE CHECK (BWFX,BWFY,LGRDOK)
      INCLUDE 'TYPES.INC'

***   CHECK  - CHECK MESH AND ADAPT IT IF NECESSARY
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION BWFX(MAXNPT-1),BWFY(MAXNPT-1)

**    START OF EXECUTABLE CODE
*     ************************
*
*     MINIMUM SPACING IS 5 ANGSTROM
      XYMIN=0.5E-7*SECAUR/XYNOR
      LGRDOK=.TRUE.

**    CHECK X-MESH
*
*     ARE SOURCE/DRAIN CONTACT LENGTHS APPROPRIATE
      XMIN=X(NXS)-MAX(SQRT(2*DIM(US,UB-1/UT)/SUBS),2*YJS)
      XMAX=X(NXD)+MAX(SQRT(2*DIM(UD,UB-1/UT)/SUBS),2*YJD)
      NOLD=NX

      H=0.5E4*XYNOR
      IF (XMIN.LT.X(1)) THEN
         WRITE (NUMREF,101)
         WRITE (NUMREF,102) 'X',0,2*H*XMIN
         LADAPT=.TRUE.
      ELSE
         LADAPT=.FALSE.
      ENDIF
      DO 1 I=1,NX-1
      IF (BWFX(I).GT.1) THEN
         IF (.NOT.LADAPT) THEN
            WRITE (NUMREF,101)
            LADAPT=.TRUE.
         ENDIF
         WRITE (NUMREF,102) 'X',I,H*(X(I+1)+X(I)),2*H*DX(I),BWFX(I)
      ENDIF
 1    CONTINUE
      IF (XMAX.GT.X(NX)) THEN
         IF (.NOT.LADAPT) THEN
            WRITE (NUMREF,101)
            LADAPT=.TRUE.
         ENDIF
         WRITE (NUMREF,102) 'X',NX+1,2*H*XMAX
      ENDIF

      CALL CHECKZ (X,DX,XMIN,XMAX,XYMIN,BWFX,NX,MAXNPT,SECAUR)
**    HAS X-MESH BEEN CHANGED
      IF (NX.NE.NOLD) LGRDOK=.FALSE.

**    CHECK Y-MESH
*
*     IS DEPTH OF SUBSTRATE APPROPRIATE
      YMIN=Y(1)
      IF (LBOTI) THEN
         YMAX=Y(NYTOT)
      ELSE
         YMAX=MAX(YJS+1.5*SQRT(2*DIM(US,UB-1/UT)/SUBS),3*YJS,
     1            YJD+1.5*SQRT(2*DIM(UD,UB-1/UT)/SUBS),3*YJD)
      ENDIF
      NOLD=NYTOT

      DO 2 J=1,NYTOT-1
      IF (BWFY(J).GT.1) THEN
         IF (.NOT.LADAPT) THEN
            WRITE (NUMREF,101)
            LADAPT=.TRUE.
         ENDIF
         WRITE (NUMREF,102) 'Y',J,H*(Y(J+1)+Y(J)),2*H*DY(J),BWFY(J)
      ENDIF
 2    CONTINUE
      IF (YMAX.GT.Y(NYTOT)) THEN
         IF (.NOT.LADAPT) THEN
            WRITE (NUMREF,101)
            LADAPT=.TRUE.
         ENDIF
         WRITE (NUMREF,102) 'Y',NYTOT+1,2*H*YMAX
      ENDIF
      IF (LADAPT) WRITE (NUMREF,103)

      CALL CHECKZ (Y,DY,YMIN,YMAX,XYMIN,BWFY,NYTOT,MAXNPT,SECAUR)
**    HAS Y-MESH BEEN CHANGED
      IF (NYTOT.NE.NOLD) LGRDOK=.FALSE.

 101  FORMAT(/11X,'START OF GRID ADAPTION'/
     1        11X,'D  I      Z         DZ      BWF')
 102  FORMAT(11X,A,I3,2F10.4,1PE10.2)
 103  FORMAT(11X,' END  OF GRID ADAPTION'/)
      END
      SUBROUTINE CHECKZ (Z,DZ,ZMIN,ZMAX,DZMIN,BWFZ,NZ,NZMAX,SECAUR)
      INCLUDE 'TYPES.INC'

***   CHECKZ - CHECK/ADAPT MESH FOR QUASI-UNIFORMITY
*
*     S.SELBERHERR   1979-1986
*
      PARAMETER (MAXINS=10)
      DIMENSION Z(NZMAX),DZ(NZMAX-1),BWFZ(NZMAX-1)

**    START OF EXECUTABLE CODE
*     ************************
*
*     COUNT OF INSERTED POINTS
      NINS=0
**    RETURN (CHECK BOUNDARIES) IF TOO MANY POINTS
 1    IF (NZ.EQ.NZMAX.OR.NINS.GE.MAXINS) GOTO 10
      BWFMAX=0
      DO 2 I=1,NZ-1
      IF (BWFZ(I).GT.BWFMAX) THEN
         IMAX=I
         BWFMAX=BWFZ(I)
      ENDIF
 2    CONTINUE

**    IF GRID IS ALREADY APPROPRIATE
      IF (BWFMAX.LE.1) GOTO 10

**    CHECK IF INTERVALL IS TO SMALL
      IF (DZ(IMAX).GT.DZMIN) GOTO 3
      BWFZ(IMAX)=0
      GOTO 1

**    INSERT AT MAXIMUM OF BWFZ
 3    IF (IMAX.EQ.1.OR.(ABS(BWFZ(MAX(1,IMAX-1))).LT.
     1ABS(BWFZ(MIN(NZ-1,IMAX+1))).AND.IMAX.NE.NZ-1)) THEN
         WSP=SECAUR-1
      ELSE
         WSP=2-SECAUR
      ENDIF
      ISP=IMAX
      JUMP=1
      GOTO 8

**    ASSURE QUASI-UNIFORMITY TO THE RIGHT
 4    K=IMAX+1
 5    IF (NZ.EQ.NZMAX) GOTO 16
      IF (K.GE.NZ-1) GOTO 6
      IF (DZ(K+1).LE.MAX(1.001*SECAUR*DZ(K),DZMIN)) GOTO 6
      WSP=2-SECAUR
      ISP=K+1
      JUMP=2
      GOTO 8

**    ASSURE QUASI-UNIFORMITY TO THE LEFT
 6    K=IMAX
 7    IF (NZ.EQ.NZMAX) GOTO 16
      IF (K.EQ.1) GOTO 1
      IF (DZ(K-1).LE.MAX(1.001*SECAUR*DZ(K),DZMIN)) GOTO 1
      K=K-1
      WSP=SECAUR-1
      ISP=K
      JUMP=3

**    SPLIT INTERVALL ISP APPROPRIATELY
 8    Z(NZ+1)=Z(NZ)
      DO 9 I=NZ,ISP+2,-1
      Z(I)=Z(I-1)
      DZ(I)=DZ(I-1)
 9    BWFZ(I)=BWFZ(I-1)
      Z(ISP+1)=Z(ISP)+DZ(ISP)*WSP
      DZ(ISP+1)=Z(ISP+2)-Z(ISP+1)
      DZ(ISP)=Z(ISP+1)-Z(ISP)
      H=-ABS(BWFZ(ISP))
      BWFZ(ISP+1)=H*(1-WSP)
      BWFZ(ISP)=H*WSP
      NZ=NZ+1
      NINS=NINS+1
      GOTO (4,5,7),JUMP

**    CHECK LEFT BOUNDARY
 10   IF (NZ.EQ.NZMAX) GOTO 13
      IF (Z(1).LE.ZMIN+DZMIN) GOTO 12
      Z(NZ+1)=Z(NZ)
      DO 11 I=NZ,2,-1
      Z(I)=Z(I-1)
      DZ(I)=DZ(I-1)
 11   BWFZ(I)=BWFZ(I-1)
      Z(1)=Z(1)-SECAUR*DZ(1)
      DZ(1)=Z(2)-Z(1)
      BWFZ(1)=0
      NZ=NZ+1
      GOTO 10

**    CHECK RIGHT BOUNDARY
 12   IF (NZ.EQ.NZMAX) GOTO 13
      IF (Z(NZ).GE.ZMAX-DZMIN) GOTO 13
      Z(NZ+1)=Z(NZ)+SECAUR*DZ(NZ-1)
      DZ(NZ)=Z(NZ+1)-Z(NZ)
      BWFZ(NZ)=0
      NZ=NZ+1
      GOTO 12

**    CHECK IF TOO MANY POINTS AT THE BOUNDARIES
 13   IF (Z(2).GT.ZMIN-DZMIN) GOTO 15
      DO 14 I=2,NZ-1
      Z(I-1)=Z(I)
      DZ(I-1)=DZ(I)
 14   BWFZ(I-1)=BWFZ(I)
      Z(NZ-1)=Z(NZ)
      NZ=NZ-1
      GOTO 13

 15   IF (Z(NZ-1).LT.ZMAX+DZMIN) GOTO 16
      NZ=NZ-1
      GOTO 15

 16   RETURN
      END
      SUBROUTINE CONCNT (CDON,CACC,FX,FY,HLP1,HLP2,HLP3)
      INCLUDE 'TYPES.INC'

***   CONCNT - CALCULATE DOPING PROFILE
*
*     A.SCHUETZ      1980-1983
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION CDON(NXY),CACC(NXY),
C    1FX(MAX(MAXDO2,NX)),FY(MAX(MAXDO2,NY)),
     1FX(*),FY(*),
     2HLP1(MAXDO2*MAXDO2),HLP2(MAXDO2*MAXDO2),HLP3(MAXDO2*MAXDO2)

      OEXP(Z)=EXP(SIGN(MIN(ABS(Z),EXPLIM),Z))
      OMEG(Z)=OEXP(-A1*(Z-RP)**2)*OERFC(-A2-A3*Z)

**    START OF EXECUTABLE CODE
*     ************************
*
      JY=NYOX-1
**    CHECK FOR DOPING ON FILE
      IF (IFILE.EQ.0) THEN
**       ANALYTICAL FORMULAE

**       SUBSTRATE
         IF (ICHANL.GT.0) THEN
            DO 7701 L=1,NXY
            CACC(L)=SUBS
 7701       CDON(L)=0
         ELSE
            DO 7702 L=1,NXY
            CACC(L)=0
 7702       CDON(L)=SUBS
         ENDIF

         IF (IPROF.EQ.0) THEN
**          SOURCE/DRAIN PREDEPOSITION
            XI=2*SQRT(DIFCOF((3+ICHANL)/2,TEMP)*TIME)/XYNOR
            DO 1 J=1,NY
            H1=(MAX(ZERO,Y(J+JY)-YOFF)/XI)**2
            IF (H1.GE.EXPLIM) GOTO 2
 1          FY(J)=ABS(SURF)/2*EXP(-H1)
 2          DO 3 JJ=J,NY
 3          FY(JJ)=0
            DO 4 I=1,NX
 4          FX(I)=OERFC(X(I)/XI)+OERFC((XLPRO1-X(I))/XI)
            CALL DOP0DA (CDON,CACC,FX,FY,ICHANL.GT.0,.TRUE.)
         ELSE
**          SOURCE IMPLANTATION (AT LEAST)
            CALL IMPPAR (ISDELE(1),SDDOSE,SDAKEV,
     1      1,SDTOX,RP,DELTRP,CMAX)
            XI=2*SQRT(DIFCOF(ISDELE(1),TEMP)*TIME)/XYNOR
            A1=1/(XI**2+2*DELTRP**2)
            H1=SQRT(2*A1)
            A2=RP/2*H1*XI/DELTRP
            A3=H1*DELTRP/XI
            H=H1/4*CMAX*DELTRP
            DO 5 J=1,NY
            H1=MAX(ZERO,Y(J+JY)-YOFF)
 5          FY(J)=(OMEG(H1)+OMEG(-H1))*H
            DO 6 I=1,NX
 6          FX(I)=OERFC(X(I)/XI)
            CALL DOP0DA (CDON,CACC,FX,FY,ISDELE(1).NE.1,.TRUE.)
         ENDIF

**       DRAIN PROFILE HAS TO BE TREATED DIFFERENTLY (EVENTUALLY)
         IF (IPROF.NE.0) THEN
            DO 7 I=1,NX
 7          FX(I)=OERFC((XLPRO1-X(I))/XI)
            IF (IPROF.EQ.2) THEN
**             DRAIN PREDEPOSITION
               XI=2*SQRT(DIFCOF((3+ICHANL)/2,TEMP)*TIME)/XYNOR
               DO 8 J=1,NY
               H1=(MAX(ZERO,Y(J+JY)-YOFF)/XI)**2
               IF (H1.GE.EXPLIM) GOTO 9
 8             FY(J)=ABS(SURF)/2*EXP(-H1)
 9             DO 10 JJ=J,NY
 10            FY(JJ)=0
               CALL DOP0DA (CDON,CACC,FX,FY,ICHANL.GT.0,.TRUE.)
            ELSEIF (IPROF.EQ.3) THEN
**             DRAIN IMPLANTATION
               CALL IMPPAR (ISDELE(2),SDDOSE(2),SDAKEV(2),
     1         1,SDTOX,RP,DELTRP,CMAX)
               XI=2*SQRT(DIFCOF(ISDELE(2),TEMP)*TIME)/XYNOR
               A1=1/(XI**2+2*DELTRP**2)
               H1=SQRT(2*A1)
               A2=RP/2*H1*XI/DELTRP
               A3=H1*DELTRP/XI
               H=H1/4*CMAX*DELTRP
               DO 11 J=1,NY
               H1=MAX(ZERO,Y(J+JY)-YOFF)
 11            FY(J)=(OMEG(H1)+OMEG(-H1))*H
               CALL DOP0DA (CDON,CACC,FX,FY,ISDELE(2).NE.1,.TRUE.)
            ELSE
               CALL DOP0DA (CDON,CACC,FX,FY,ISDELE(1).NE.1,.TRUE.)
            ENDIF
         ENDIF

         IF (IPROF.GE.4) THEN
            IF (IPROF.LE.5) THEN
               DO 12 I=1,NX
 12            FX(I)=OERFC((XLPRO2-X(I))/XI)+OERFC((XOFF+X(I))/XI)
            ELSE
               DO 13 I=1,NX
 13            FX(I)=OERFC((XLPRO2-X(I))/XI)
            ENDIF

            IF (IPROF.EQ.4.OR.IPROF.EQ.6) THEN
**             HEAVILY DOPED PREDEPOSITION
               XI=2*SQRT(DIFCOF((3+ICHANL)/2,TEMP)*TIME)/XYNOR
               DO 14 J=1,NY
               H1=(MAX(ZERO,Y(J+JY)-YOFF)/XI)**2
               IF (H1.GE.EXPLIM) GOTO 15
 14            FY(J)=ABS(SURF)/2*EXP(-H1)
 15            DO 16 JJ=J,NY
 16            FY(JJ)=0
               CALL DOP0DA (CDON,CACC,FX,FY,ICHANL.GT.0,.TRUE.)
            ELSE
**             HEAVILY DOPED IMPLANTATION
               CALL IMPPAR (ISDELE(2),SDDOSE(2),SDAKEV(2),
     1         1,SDTOX,RP,DELTRP,CMAX)
               XI=2*SQRT(DIFCOF(ISDELE(2),TEMP)*TIME)/XYNOR
               A1=1/(XI**2+2*DELTRP**2)
               H1=SQRT(2*A1)
               A2=RP/2*H1*XI/DELTRP
               A3=H1*DELTRP/XI
               H=H1/4*CMAX*DELTRP
               DO 17 J=1,NY
               H1=MAX(ZERO,Y(J+JY)-YOFF)
 17            FY(J)=(OMEG(H1)+OMEG(-H1))*H
               CALL DOP0DA (CDON,CACC,FX,FY,ISDELE(2).NE.1,.TRUE.)
            ENDIF
         ENDIF

**       CHANNEL IMPLANTATION
         DO 19 IMPL=1,2
         IELEM=ICHELE(IMPL)
         IF (IELEM.GE.1) THEN
            DOSE=CHDOSE(IMPL)
            AKEV=CHAKEV(IMPL)
            IF (EINS.EQ.3.9) THEN
               INS=1
            ELSE
               INS=2
            ENDIF
            CALL IMPPAR (IELEM,DOSE,AKEV,INS,TINS,RP,DELTRP,CMAX)
            XI=2*SQRT(DIFCOF(IELEM,CHTEMP)*CHTIME)/XYNOR
            A1=1/(XI**2+2*DELTRP**2)
            H1=SQRT(2*A1)
            A2=RP/2*H1*XI/DELTRP
            A3=H1*DELTRP/XI
            H=H1/2*CMAX*DELTRP
            DO 18 J=1,NY
            H1=Y(J+JY)
 18         FY(J)=(OMEG(H1)+OMEG(-H1))*H
            CALL DOP0DA (CDON,CACC,FX,FY,IELEM.NE.1,.FALSE.)
         ENDIF
 19      CONTINUE
      ELSEIF (IFILE.EQ.1) THEN
         CALL DOP1D (CDON,CACC,HLP1,HLP2)
      ELSE
         CALL DOP2D (CDON,CACC,HLP1,HLP2,HLP3,FX,FY)
      ENDIF

**    NOW ENSURE THAT PROFILE IS CONSISTENT
      DO 20 L=1,NXY
      CDON(L)=MAX(ZERO,CDON(L))
 20   CACC(L)=MAX(ZERO,CACC(L))

      END
      FUNCTION DIFCOF (IELEM,TEMP)
      INCLUDE 'TYPES.INC'

***   DIFCOF - CALCULATE INTRINSIC DIFFUSION COEFFICIENT
*
*     S.SELBERHERR   1979-1986
*

***   ON ENTRY
*
*     IELEM  - ELEMENT INDEX
*              1 = BORON
*              2 = PHOSPHORUS
*              3 = ANTIMONY
*              4 = ARSENIC
*
*     TEMP   - DIFFUSION TEMPERATURE
*

***   ON EXIT
*
*     DIFCOF - DIFFUSION COEFFICIENT
*
      DIMENSION DIF0(4),DIFA(4)
      DATA DIF0 /0.5554,3.85,12.9,24./
      DATA DIFA /-3.975E4,-4.247E4,-4.619E4,-4.735E4/

      DIFCOF=DIF0(IELEM)*EXP(DIFA(IELEM)/(TEMP+273.16))

      END
      SUBROUTINE DOP0DA (CDON,CACC,FX,FY,LDON,L2D)
      INCLUDE 'TYPES.INC'

***   DOP0DA - ACCUMULATE ANALYTICAL DOPING PROFILE
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION CDON(NX,NY),CACC(NX,NY),FX(NX),FY(NY)

**    START OF EXECUTABLE CODE
*     ************************
*
      IF (L2D) THEN
         IF (LDON) THEN
            DO 1 J=1,NY
            DO 1 I=1,NX
 1          CDON(I,J)=CDON(I,J)+FY(J)*FX(I)
         ELSE
            DO 2 J=1,NY
            DO 2 I=1,NX
 2          CACC(I,J)=CACC(I,J)+FY(J)*FX(I)
         ENDIF
      ELSE
         IF (LDON) THEN
            DO 3 J=1,NY
            DO 3 I=1,NX
 3          CDON(I,J)=CDON(I,J)+FY(J)
         ELSE
            DO 4 J=1,NY
            DO 4 I=1,NX
 4          CACC(I,J)=CACC(I,J)+FY(J)
         ENDIF
      ENDIF

      END
      SUBROUTINE FIXJ (C)
      INCLUDE 'TYPES.INC'

***   FIXJ - FIND VARIOUS JUNCTION DEPTHS
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION C(NX,NY)

**    START OF EXECUTABLE CODE
*     ************************
*
*     FIND SUBDIFFUSION AT SOURCE SIDE
      DO 1 J=1,NY
      DO 1 I=NXS,NXG2
      IF (C(I,J).LT.0) GOTO 2
 1    CONTINUE
      GOTO 13

 2    IF (C(I-1,J).LT.0) GOTO 13
      XJS=X(I-1)-DX(I-1)*C(I-1,J)/(C(I,J)-C(I-1,J))
      NXJS=I

**    FIND SUBDIFFUSION AT DRAIN SIDE
      DO 3 J=1,NY
      DO 3 I=NXD,NXG1,-1
      IF (C(I,J).LT.0) GOTO 4
 3    CONTINUE
      GOTO 13

 4    IF (C(I+1,J).LT.0) GOTO 13
      XJD=XLGEO-(X(I)-DX(I)*C(I,J)/(C(I+1,J)-C(I,J)))
      NXJD=I

**    FIND JUNCTION DEPTH AT SOURCE SIDE
      DO 5 J=1,NY
      IF (C(NXS,J).LT.0) GOTO 6
 5    CONTINUE
      IF (.NOT.LBOTI) GOTO 13
      YJS=YSEMI
      NYJS=NY
      GOTO 7

 6    YJS=Y(J+NYOX-2)-DY(J+NYOX-2)*C(NXS,J-1)/(C(NXS,J)-C(NXS,J-1))
      NYJS=J-1

**    FIND JUNCTION DEPTH AT DRAIN SIDE
 7    DO 8 J=1,NY
      IF (C(NXD,J).LT.0) GOTO 9
 8    CONTINUE
      IF (.NOT.LBOTI) GOTO 13
      YJD=YSEMI
      NYJD=NY
      GOTO 10

 9    YJD=Y(J+NYOX-2)-DY(J+NYOX-2)*C(NXD,J-1)/(C(NXD,J)-C(NXD,J-1))
      NYJD=J-1

**    FIND ACTIVE PART OF CHANNEL FOR MODEL=1-D
 10   IF (LBOTI) THEN
         NT=NY
      ELSE
         DO 11 J=1,NY
         IF (Y(J+NYOX-1).GT.YJS/2) GOTO 12
 11      CONTINUE
         GOTO 13
 12      NT=J
      ENDIF
      NXT=NX*NT
      NXTMX=NXT-NX
      GOTO 14

 13   IFAIL=1
 14   RETURN
      END
      SUBROUTINE IMPPAR (IELEM,DOSE,AKEV,IOX,TIOX,RP,DRP,CMAX)
      INCLUDE 'TYPES.INC'

***   IMPPAR - ESTIMATE IMPLANTATION PARAMETERS
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      CALL IMPSTA (IELEM,1,AKEV,RP,DRP,DUMMY1,DUMMY2,DUMMY3)
      CALL IMPSTA (IELEM,IOX+1,AKEV,RPOX,DUMMY1,DUMMY2,DUMMY3,DUMMY4)

**    ADJUST MICRON TO CENTIMETER
      RP=RP*1.E-4
      DRP=DRP*1.E-4
      RPOX=RPOX*1.E-4
**    1/SQRT(2*PI)=0.39894228
      CMAX=0.39894228*DOSE/(DRP*CNOR)
      DRP=DRP/XYNOR
      RP=RP*(1-TIOX/(RPOX/XYNOR))/XYNOR

      END
      SUBROUTINE IMPSTA (IELEM,ISUBS,AKEV,RP,DRP,SKE,RKU,DRPL)
      INCLUDE 'TYPES.INC'

***   IMPSTA - CALCULATE IMPLANTATION PARAMETERS
*
*     S.SELBERHERR   1988
*

***   ON ENTRY
*
*     IELEM  - IMPLANTATION ELEMENT INDEX
*              IF ISUBS =1, 2, 3 THEN
*                 1 = BORON
*                 2 = PHOSPHORUS
*                 3 = ANTIMONY
*                 4 = ARSENIC
*              IF ISUBS = 4 THEN
*                 1 = BERYLLIUM
*                 2 = BORON
*                 3 = OXYGEN
*                 4 = MAGNESIUM
*                 5 = SILICON
*                 6 = PHOSPHORUS
*                 7 = ARSENIC
*
*     ISUBS  - SUBSTRATE INDEX
*              1 = SILICON
*              2 = SILICON DIOXIDE
*              3 = SILICON NITRIDE
*              4 = GALLIUM ARSENIDE
*
*     AKEV   - IMPLANTATION ENERGY IN KEV
*              0 .LT. AKEV .LT. 300
*

***   ON EXIT
*
*     RP     - PROJECTED RANGE IN MICRONS
*
*     DRP    - PROJECTED STANDARD DEVIATION IN MICRONS
*
*     SKE    - SKEWNESS
*
*     RKU    - KURTOSIS
*
*     DRPL   - LATERAL PROJECTED STANDARD DEVIATION IN MICRONS
*
      DIMENSION CRP(19,5),CDRP(19,5),CSKE(19,4),CKUR(19,5),CDRL(19,5)

***   PROJECTED RANGE
*
*     B  IN SI,    MSE=0.339%
      DATA (CRP( 1,L),L=1,5) / 8.84, -5.693, 0.3470,  70.31, 0.836/
*     P  IN SI,    MSE=0.203%
      DATA (CRP( 2,L),L=1,5) / 3.44, -7.274, 0.6979,  13.58, 0.966/
*     SB IN SI,    MSE=0.415%
      DATA (CRP( 3,L),L=1,5) / 3.76,-10.022, 0.8748,  10.50, 0.842/
*     AS IN SI,    MSE=0.420%
      DATA (CRP( 4,L),L=1,5) / 4.40, -6.880, 0.5171,   8.92, 0.937/
*     B  IN SIO2,  MSE=0.356%
      DATA (CRP( 5,L),L=1,5) / 9.56, -4.428, 0.2687,  83.17, 0.815/
*     P  IN SIO2,  MSE=0.163%
      DATA (CRP( 6,L),L=1,5) / 2.96, -8.429, 0.9428,  12.98, 0.979/
*     SB IN SIO2,  MSE=0.429%
      DATA (CRP( 7,L),L=1,5) / 3.75,-10.241, 0.8979,   9.46, 0.852/
*     AS IN SIO2,  MSE=0.448%
      DATA (CRP( 8,L),L=1,5) / 4.40, -6.706, 0.5052,   7.86, 0.952/
*     B  IN SI3N4, MSE=0.352%
      DATA (CRP( 9,L),L=1,5) / 9.52, -3.707, 0.2230,  67.73, 0.808/
*     P  IN SI3N4, MSE=0.166%
      DATA (CRP(10,L),L=1,5) / 2.99, -6.915, 0.7666,  10.79, 0.972/
*     SB IN SI3N4, MSE=0.430%
      DATA (CRP(11,L),L=1,5) / 3.68, -9.084, 0.8117,   7.92, 0.849/
*     AS IN SI3N4, MSE=0.440%
      DATA (CRP(12,L),L=1,5) / 4.20, -6.142, 0.4840,   6.58, 0.947/
*     BE IN GAAS,  MSE=0.314%
      DATA (CRP(13,L),L=1,5) / 7.98, -7.862, 0.5010,  66.86, 0.829/
*     B  IN GAAS,  MSE=0.293%
      DATA (CRP(14,L),L=1,5) / 8.93, -6.656, 0.3901,  60.33, 0.816/
*     O  IN GAAS,  MSE=0.285%
      DATA (CRP(15,L),L=1,5) / 7.76, -5.746, 0.3633,  29.36, 0.873/
*     MG IN GAAS,  MSE=0.285%
      DATA (CRP(16,L),L=1,5) / 4.05, -4.868, 0.3451,  11.43, 1.020/
*     SI IN GAAS,  MSE=0.267%
      DATA (CRP(17,L),L=1,5) / 4.20, -5.295, 0.4202,  10.17, 1.003/
*     P  IN GAAS,  MSE=0.253%
      DATA (CRP(18,L),L=1,5) / 3.93, -5.445, 0.4583,  10.41, 0.964/
*     AS IN GAAS,  MSE=0.516%
      DATA (CRP(19,L),L=1,5) / 6.08, -3.224, 0.1935,   5.51, 0.951/

***   VERTICAL STANDARD DEVIATION
*
*     B  IN SI,    MSE=0.436%
      DATA (CDRP( 1,L),L=1,5) / 9.25, -3.802, 0.2142, 229.43, 0.331/
*     P  IN SI,    MSE=0.324%
      DATA (CDRP( 2,L),L=1,5) / 6.99, -2.419, 0.1623,  16.21, 0.689/
*     SB IN SI,    MSE=0.332%
      DATA (CDRP( 3,L),L=1,5) / 4.19, -1.729, 0.1336,   3.58, 0.816/
*     AS IN SI,    MSE=0.248%
      DATA (CDRP( 4,L),L=1,5) / 3.70, -2.431, 0.2132,   4.68, 0.849/
*     B  IN SIO2,  MSE=0.454%
      DATA (CDRP( 5,L),L=1,5) / 9.82, -3.058, 0.1672, 333.86, 0.279/
*     P  IN SIO2,  MSE=0.350%
      DATA (CDRP( 6,L),L=1,5) / 7.64, -1.932, 0.1223,  14.95, 0.702/
*     SB IN SIO2,  MSE=0.334%
      DATA (CDRP( 7,L),L=1,5) / 4.14, -1.403, 0.1099,   2.88, 0.826/
*     AS IN SIO2,  MSE=0.297%
      DATA (CDRP( 8,L),L=1,5) / 4.31, -1.551, 0.1170,   3.55, 0.874/
*     B  IN SI3N4, MSE=0.438%
      DATA (CDRP( 9,L),L=1,5) / 9.78, -2.503, 0.1363, 275.66, 0.269/
*     P  IN SI3N4, MSE=0.351%
      DATA (CDRP(10,L),L=1,5) / 7.56, -1.598, 0.1016,  12.40, 0.693/
*     SB IN SI3N4, MSE=0.347%
      DATA (CDRP(11,L),L=1,5) / 3.81, -1.317, 0.1120,   2.48, 0.820/
*     AS IN SI3N4, MSE=0.289%
      DATA (CDRP(12,L),L=1,5) / 4.18, -1.342, 0.1043,   3.04, 0.866/
*     BE IN GAAS,  MSE=0.473%
      DATA (CDRP(13,L),L=1,5) / 9.35, -5.201, 0.2927, 233.42, 0.386/
*     B  IN GAAS,  MSE=0.503%
      DATA (CDRP(14,L),L=1,5) /10.00, -4.586, 0.2498, 236.15, 0.370/
*     O  IN GAAS,  MSE=0.476%
      DATA (CDRP(15,L),L=1,5) / 9.50, -3.496, 0.1931,  76.13, 0.508/
*     MG IN GAAS,  MSE=0.202%
      DATA (CDRP(16,L),L=1,5) / 3.06, -8.175, 1.1377,  13.61, 0.821/
*     SI IN GAAS,  MSE=0.150%
      DATA (CDRP(17,L),L=1,5) / 2.75, -7.509, 1.0862,  11.58, 0.813/
*     P  IN GAAS,  MSE=0.183%
      DATA (CDRP(18,L),L=1,5) / 3.08, -7.071, 0.9764,  12.22, 0.762/
*     AS IN GAAS,  MSE=0.290%
      DATA (CDRP(19,L),L=1,5) / 3.41, -3.084, 0.2942,   4.46, 0.844/

***   SKEWNESS
*
*     B  IN SI,    MSE=0.0026%
      DATA (CSKE( 1,L),L=1,4) / 1.534, 0.03186,-1.191,-0.00259/
*     P  IN SI,    MSE=0.0008%
      DATA (CSKE( 2,L),L=1,4) / 0.822, 0.01667,-0.441,-0.00162/
*     SB IN SI,    MSE=0.0017%
      DATA (CSKE( 3,L),L=1,4) / 0.000,-0.05047, 0.484,-0.00077/
*     AS IN SI,    MSE=0.0011%
      DATA (CSKE( 4,L),L=1,4) / 0.675, 0.00506,-0.121,-0.00024/
*     B  IN SIO2,  MSE=0.0036%
      DATA (CSKE( 5,L),L=1,4) / 1.756, 0.02623,-1.362,-0.00203/
*     P  IN SIO2,  MSE=0.0006%
      DATA (CSKE( 6,L),L=1,4) / 0.998, 0.01295,-0.572,-0.00122/
*     SB IN SIO2,  MSE=0.0012%
      DATA (CSKE( 7,L),L=1,4) /-1.066, 8.01228, 0.410,-0.00063/
*     AS IN SIO2,  MSE=0.0021%
      DATA (CSKE( 8,L),L=1,4) / 1.409, 0.00176,-0.953, 0.00026/
*     B  IN SI3N4, MSE=0.0027%
      DATA (CSKE( 9,L),L=1,4) / 1.608, 0.03202,-1.234,-0.00256/
*     P  IN SI3N4, MSE=0.0009%
      DATA (CSKE(10,L),L=1,4) / 0.878, 0.01381,-0.504,-0.00151/
*     SB IN SI3N4, MSE=0.0018%
      DATA (CSKE(11,L),L=1,4) /-1.007, 4.76355, 0.388,-0.00063/
*     AS IN SI3N4, MSE=0.0030%
      DATA (CSKE(12,L),L=1,4) / 0.059, 0.01098, 0.349,-0.00122/
*     BE IN GAAS,  MSE=0.0022%
      DATA (CSKE(13,L),L=1,4) / 1.365, 0.02546,-0.742,-0.00167/
*     B  IN GAAS,  MSE=0.0018%
      DATA (CSKE(14,L),L=1,4) / 1.421, 0.01975,-0.783,-0.00119/
*     O  IN GAAS,  MSE=0.0015%
      DATA (CSKE(15,L),L=1,4) / 1.266, 0.01568,-0.614,-0.00102/
*     MG IN GAAS,  MSE=0.0011%
      DATA (CSKE(16,L),L=1,4) / 0.771, 0.01704,-0.032,-0.00132/
*     SI IN GAAS,  MSE=0.0008%
      DATA (CSKE(17,L),L=1,4) / 0.799, 0.01414,-0.077,-0.00114/
*     P  IN GAAS,  MSE=0.0009%
      DATA (CSKE(18,L),L=1,4) / 0.850, 0.01331,-0.178,-0.00104/
*     AS IN GAAS,  MSE=0.0012%
      DATA (CSKE(19,L),L=1,4) / 1.042, 0.00232,-0.402, 0.00007/

***   KURTOSIS
*
*     B  IN SI,    MSE=0.245%
      DATA (CKUR( 1,L),L=1,4) / 0.995, 1.59400, 2.236, 0.02099/
*     P  IN SI,    MSE=0.050%
      DATA (CKUR( 2,L),L=1,4) / 0.757, 0.03632, 2.064, 0.00514/
*     SB IN SI,    MSE=0.115%
      DATA (CKUR( 3,L),L=1,4) / 0.896, 0.00337, 2.347, 0.00022/
*     AS IN SI,    MSE=0.091%
      DATA (CKUR( 4,L),L=1,4) / 0.855, 0.01068, 2.398, 0.00032/
*     B  IN SIO2,  MSE=0.454%
      DATA (CKUR( 5,L),L=1,4) / 1.475, 2.21952, 2.304, 0.02093/
*     P  IN SIO2,  MSE=0.068%
      DATA (CKUR( 6,L),L=1,4) / 0.901, 0.03882, 2.108, 0.00473/
*     SB IN SIO2,  MSE=0.172%
      DATA (CKUR( 7,L),L=1,4) / 0.432, 0.00140, 2.689,-0.00038/
*     AS IN SIO2,  MSE=0.086%
      DATA (CKUR( 8,L),L=1,4) / 2.882, 0.00237, 0.275, 0.00262/
*     B  IN SI3N4, MSE=0.313%
      DATA (CKUR( 9,L),L=1,4) / 1.196, 1.84591, 2.286, 0.02243/
*     P  IN SI3N4, MSE=0.076%
      DATA (CKUR(10,L),L=1,4) / 0.859, 0.03490, 2.072, 0.00530/
*     SB IN SI3N4, MSE=0.189%
      DATA (CKUR(11,L),L=1,4) / 0.382, 0.00176, 2.670,-0.00016/
*     AS IN SI3N4, MSE=0.117%
      DATA (CKUR(12,L),L=1,4) /10.635, 0.00085,-7.541, 0.00607/
*     BE IN GAAS,  MSE=0.110%
      DATA (CKUR(13,L),L=1,4) / 1.289, 0.14546, 1.868, 0.00856/
*     B  IN GAAS,  MSE=0.104%
      DATA (CKUR(14,L),L=1,4) / 1.304, 0.12796, 1.915, 0.00648/
*     O  IN GAAS,  MSE=0.070%
      DATA (CKUR(15,L),L=1,4) / 1.326, 0.06644, 1.885, 0.00433/
*     MG IN GAAS,  MSE=0.070%
      DATA (CKUR(16,L),L=1,4) / 1.417, 0.03002, 1.983, 0.00158/
*     SI IN GAAS,  MSE=0.053%
      DATA (CKUR(17,L),L=1,4) / 1.432, 0.02205, 1.906, 0.00169/
*     P  IN GAAS,  MSE=0.059%
      DATA (CKUR(18,L),L=1,4) / 1.298, 0.02478, 1.936, 0.00197/
*     AS IN GAAS,  MSE= 0.082%
      DATA (CKUR(19,L),L=1,4) / 0.704, 0.01020, 2.641,-0.00032/

***   LATERAL STANDARD DEVIATION
*
*     B  IN SI,    MSE=0.445%
      DATA (CDRL( 1,L),L=1,5) / 9.01, -4.105, 0.2375, 120.73, 0.476/
*     P  IN SI,    MSE=0.158%
      DATA (CDRL( 2,L),L=1,5) / 2.87, -6.088, 0.8788,   8.85, 0.789/
*     SB IN SI,    MSE=0.375%
      DATA (CDRL( 3,L),L=1,5) / 3.92, -2.258, 0.1874,   3.03, 0.780/
*     AS IN SI,    MSE=0.348%
      DATA (CDRL( 4,L),L=1,5) / 4.19, -2.268, 0.1805,   3.35, 0.852/
*     B  IN SIO2,  MSE=0.449%
      DATA (CDRL( 5,L),L=1,5) / 9.65, -3.138, 0.1767, 149.25, 0.442/
*     P  IN SIO2,  MSE=0.203%
      DATA (CDRL( 6,L),L=1,5) / 3.02, -4.821, 0.6683,   7.35, 0.812/
*     SB  IN SIO2, MSE=0.384%
      DATA (CDRL( 7,L),L=1,5) / 3.96, -1.969, 0.1624,   2.44, 0.785/
*     AS IN SIO2,  MSE=0.374%
      DATA (CDRL( 8,L),L=1,5) / 4.17, -1.901, 0.1498,   2.52, 0.871/
*     B  IN SI3N4, MSE=0.453%
      DATA (CDRL( 9,L),L=1,5) / 9.58, -2.556, 0.1434, 118.67, 0.436/
*     P  IN SI3N4, MSE=0.196%
      DATA (CDRL(10,L),L=1,5) / 3.10, -4.106, 0.5650,   6.19, 0.802/
*     SB IN SI3N4, MSE=0.398%
      DATA (CDRL(11,L),L=1,5) / 4.06, -1.680, 0.1352,   2.02, 0.785/
*     AS IN SI3N4, MSE=0.393%
      DATA (CDRL(12,L),L=1,5) / 4.12, -1.673, 0.1334,   2.16, 0.864/
*     BE IN GAAS,  MSE=0.416%
      DATA (CDRL(13,L),L=1,5) / 8.90, -6.863, 0.3978, 191.34, 0.461/
*     B  IN GAAS,  MSE=0.432%
      DATA (CDRL(14,L),L=1,5) / 9.62, -5.775, 0.3202, 182.25, 0.451/
*     O  IN GAAS,  MSE=0.417%
      DATA (CDRL(15,L),L=1,5) / 8.95, -4.373, 0.2518,  61.71, 0.578/
*     MG IN GAAS,  MSE=0.151%
      DATA (CDRL(16,L),L=1,5) / 2.01,-10.812, 1.7930,  12.42, 0.862/
*     SI IN GAAS,  MSE=0.117%
      DATA (CDRL(17,L),L=1,5) / 1.71, -7.494, 0.8593,  10.39, 0.851/
*     P  IN GAAS,  MSE=0.109%
      DATA (CDRL(18,L),L=1,5) / 1.98, -9.716, 1.6754,  10.26, 0.808/
*     AS IN GAAS,  MSE=0.372%
      DATA (CDRL(19,L),L=1,5) / 3.92, -2.620, 0.2193,   3.38, 0.865/

      X=AKEV
**    CALCULATE INTERNAL INDEX
      I=(ISUBS-1)*4+IELEM

**    PROJECTED RANGE
      Y=CRP(I,1)
      B=CRP(I,2)
      C=CRP(I,3)
      D=CRP(I,4)
      P=CRP(I,5)
      E=D*Y**P
      A=P*E/Y-Y*(2*B+3*C*Y)
      IF (X.LE.Y) THEN
         RP=X*(A+X*(B+X*C))
      ELSE
         RP=D*X**P+Y*(A+Y*(B+Y*C))-E
      ENDIF

**    PROJECTED STANDARD DEVIATION
      Y=CDRP(I,1)
      B=CDRP(I,2)
      C=CDRP(I,3)
      D=CDRP(I,4)
      P=CDRP(I,5)
      E=D*Y**P
      A=P*E/Y-Y*(2*B+3*C*Y)
      IF (X.LE.Y) THEN
         DRP=X*(A+X*(B+X*C))
      ELSE
         DRP=D*X**P+Y*(A+Y*(B+Y*C))-E
      ENDIF

**    SKEWNESS
      A=CSKE(I,1)
      B=CSKE(I,2)
      C=CSKE(I,3)
      D=CSKE(I,4)
      SKE=A/(1+B*X)+C+D*X

**    KURTOSIS
      A=CKUR(I,1)
      B=CKUR(I,2)
      C=CKUR(I,3)
      D=CKUR(I,4)
      RKU=A/(1+B*X)+C+D*X

**    PROJECTED LATERAL STANDARD DEVIATION
      Y=CDRL(I,1)
      B=CDRL(I,2)
      C=CDRL(I,3)
      D=CDRL(I,4)
      P=CDRL(I,5)
      E=D*Y**P
      A=P*E/Y-Y*(2*B+3*C*Y)
      IF (X.LE.Y) THEN
         DRPL=X*(A+X*(B+X*C))
      ELSE
         DRPL=D*X**P+Y*(A+Y*(B+Y*C))-E
      ENDIF

**    SCALE TO MICRONS
      RP=RP*1.E-4
      DRP=DRP*1.E-4
      DRPL=DRPL*1.E-4

      END
      FUNCTION OERFC(X)
      INCLUDE 'TYPES.INC'

***   OERFC - CALCULATE COMPLEMENTARY ERROR FUNCTION
*
*     S.SELBERHERR   1979-1986
*
**    OSQPI=1/SQRT(PI)
      PARAMETER (OSQPI=0.564189583548,ACC=1.E-8)
      INCLUDE 'CMSYSX.INC'

      Y=ABS(X)
      Y2=Y*Y
      IF (Y.LT.2.0) THEN
         FNEW=2*OSQPI*EXP(-Y2)*Y
         H=FNEW
         R=1
         R=R/2
 1       R=R+1
         FOLD=FNEW
         H=H*Y2/R
         FNEW=FNEW+H
         IF (H.GT.ACC*FNEW) GOTO 1
         OERFC=1-SIGN(FNEW,X)
      ELSEIF (Y.LT.ERFLIM) THEN
         R=2*Y2+3
         A1=2
         A2=0
         P1=OSQPI*EXP(-Y2)
         P2=(R-1)*P1
         Q1=Y
         Q2=R*Q1
         FOLD=P1/Q1
         FNEW=P2/Q2
 2       R=R+4
         A1=A1-8
         A2=A1+A2
         H=A2*P1+R*P2
         P1=P2
         P2=H
         H=A2*Q1+R*Q2
         Q1=Q2
         Q2=H
         FVOLD=FOLD
         FOLD=FNEW
         FNEW=P2/Q2
         IF (Q2.GT.1.E10) THEN
            P1=P1*1.E-10
            P2=P2*1.E-10
            Q1=Q1*1.E-10
            Q2=Q2*1.E-10
         ENDIF
         IF (ABS(FNEW).LE.SMALL/ACC) GOTO 3
         IF (MAX(ABS(FNEW-FVOLD),ABS(FNEW-FOLD))/ACC.GT.ABS(FNEW))GOTO 2
 3       IF (X.GE.0) THEN
            OERFC=FNEW
         ELSE
            OERFC=2-FNEW
         ENDIF
      ELSE
         IF (X.GE.0) THEN
            OERFC=0
         ELSE
            OERFC=2
         ENDIF
      ENDIF

      END
