      PROGRAM MINMOS
      INCLUDE 'TYPES.INC'

***   ******************************************************************
*     *                                                                *
*     * M   M  II  N   N  II  M   M  OOOOO   SSS       4   4      000  *
*     * MM MM  II  NN  N  II  MM MM  O   O  S   S      4   4     0   0 *
*     * M M M  II  N N N  II  M M M  O   O  S          4   4     0   0 *
*     * M   M  II  N  NN  II  M   M  O   O   SSS       44444     0   0 *
*     * M   M  II  N  NN  II  M   M  O   O      S          4     0   0 *
*     * M   M  II  N   N  II  M   M  O   O  S   S          4  ** 0   0 *
*     * M   M  II  N   N  II  M   M  OOOOO   SSS           4  **  000  *
*     *                                                                *
*     ******************************************************************
*
*
*     S.SELBERHERR   1979-
*     A.SCHUETZ      1980-1983
*     P.PICHLER      1984-1985
*     W.SEMMLINGER   1985-1986
*     M.THURNER      1985-
*     J.FARICELLI    1986-
*     W.MADER        1987-
*

***   THIS PROGRAM HAS BEEN DEVELOPED AT THE
*
*     ABTEILUNG FUER PHYSIKALISCHE ELEKTRONIK
*     INSTITUT FUER ALLGEMEINE ELEKTROTECHNIK UND ELEKTRONIK
*     TECHNICAL UNIVERSITY OF VIENNA / AUSTRIA
*
*     *******************************************************
*     * THIS PROGRAM MUST NOT BE SOLD OR GIVEN AWAY WITHOUT *
*     * THE WRITTEN PERMISSION OF THE AUTHORS.              *
*     *******************************************************
*

**    SYSTEM PARAMETERS
*
*     VAR.     DESCRIPTION                                  TYPE
*     -----------------------------------------------------------
*     BERMIN - MINIMUM REAL FOR WHICH TO EVALUATE BER(X)      R
*     CPUTIM - ELAPSED CPU-TIME AT START OF EXECUTION         R
*     DATTIM - DATE AND TIME OF SIMULATION                    C
*     EPSMA  - COMPUTER PRECISION (1.+EPSMA/2.=1.)            R
*     ERFLIM - = SQRT(EXPLIM)                                 R
*     EXPLIM - = -LOG(SMALL)                                  R
*     NEROCC - COUNT OF MULTIPLE ITEM OCCURENCES              I
*     NERPHY - COUNT OF PHYSICAL ERRORS                       I
*     NERSYN - COUNT OF SYNTACTIC INPUT ERRORS                I
*     NPAGE  - CURRENT PRINTOUT PAGE NUMBER                   I
*     NUMBIN - NUMBER OF BINARY OUTPUT FILE UNIT              I PAR
*     NUMDOP - NUMBER OF DOPING INPUT FILE UNIT               I PAR
*     NUMINP - NUMBER OF INPUT FILE UNIT                      I PAR
*     NUMOUT - NUMBER OF OUTPUT FILE UNIT                     I PAR
*     NUMREF - NUMBER OF REFERENCE OUTPUT FILE UNIT           I PAR
*     SMALL  - SMALLEST POSSIBLE REAL QUANTITY                R
*
      INCLUDE 'CMSYSX.INC'

**    INPUT DIRECTIVE PARAMETER CELLS
*
*     VAR.     DESCRIPTION                                  TYPE  DIR.
*     ------------------------------------------------------------------
*     ALPHA  - IONIZATION PARAMETER CONSTANTS                 R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     AUGER  - AUGER COEFFICIENTS                             R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     BETA   - IONIZATION PARAMETER CONSTANTS                 R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     CHAKEV - CHANNEL IMPLANTATION ENERGY                    R  IMPLANT
*              ARRAY OF 2 ELEMENTS
*     CHDOSE - CHANNEL IMPLANTATION DOSES                     R  IMPLANT
*              ARRAY OF 2 ELEMENTS
*     CHTEMP - CHANNEL IMPLANTATION DIFFUSION TEMPERATURE     R  IMPLANT
*     CHTIME - CHANNEL IMPLANTATION DIFFUSION TIME            R  IMPLANT
*     CINTR  - INTRINSIC CONCENTRATION                        R  OPTION
*     CISS   - PEAK CONCENTRATION OF INTERFACE-STATES PROFILE R  INTERF.
*     DGAP   - DRAIN CONTACT GAP                              R  DEVICE
*     DUB    - BULK VOLTAGE INCREMENT                         R  STEP
*     DUD    - DRAIN VOLTAGE INCREMENT                        R  STEP
*     DUG    - GATE VOLTAGE INCREMENT                         R  STEP
*     EAMEV  - ACTIVATION ENERGY OF ACCEPTORS                 R  OPTION
*     ECMED  - ACTIVATION ENERGY OF DONORS                    R  OPTION
*     EINS   - PERMITTIVITY OF GATE INSULATOR                 R  DEVICE
*     EISUBS - PERMITTIVITY OF INSULATING SUBSTRATE           R  DEVICE
*     ERRCAP - DESIRED ACCURACY FOR CAPACITANCES              R  END
*     ERROR  - DESIRED ACCURACY FOR SPACE CHARGE              R  END
*     ERRTC  - DESIRED ACCURACY FOR CARRIER ENERGIES          R  END
*     FITL   - LATERAL FIT PARAMETER FOR S/D PROFILE          R  PROFILE
*     GATE   - GATE MATERIAL INDICATOR                        R  DEVICE
*     ICHANL - DEVICE TYPE INDICATOR                          I  DEVICE
*              -1 = P-CHANNEL
*              +1 = N-CHANNEL
*     ICHELE - CHANNEL IMPLANTATION ELEMENTS                  I  IMPLANT
*              ARRAY OF 2 ELEMENTS
*              1 = BORON
*              2 = PHOSPHORUS
*              3 = ANTIMONY
*              4 = ARSENIC
*     IFILE  - TECHNOLOGY DATA ON FILE INDEX                  I  PROFILE
*              0 = NO FILE
*              1 = 1-D FILE
*              2 = 2-D FILE
*     IPROF  - S/D PROFILE TYPE INDICATOR    (IMPLICIT INPUT) I
*              BIT 0 = 1 IF ANALYTIC S/D IMPLANTATION GIVEN
*              BIT 1 = 1 IF *ASYM* KEY GIVEN
*              BIT 2 = 1 IF *XOFF* KEY GIVEN
*     ISDELE - S/D IMPLANTATION ELEMENTS                      I  PROFILE
*              ARRAY OF 2 ELEMENTS
*              1 = BORON
*              2 = PHOSPHORUS
*              3 = ANTIMONY
*              4 = ARSENIC
*     JCAP   - NODE INDEX FOR SMALL SIGNAL BIAS               I  BIAS
*              0 =  BIAS SUCCESSIVELY AT ALL TERMINALS
*              1 =  BIAS AT GATE
*              2 =  BIAS AT SOURCE
*              3 =  BIAS AT DRAIN
*              4 =  BIAS AT BULK
*     LALL   - ALL DISTRIBUTIONS                              L  OUTPUT
*     LASYM  - NON-SYMMETRIC S/D PROFILE FLAG                 L  PROFILE
*     LAVL   - AVALANCHE GENERATION                           L  OUTPUT
*     LCAP   - SMALL SIGNAL ANALYSIS FLAG    (IMPLICIT INPUT) L
*     LCC    - CARRIER CONCENTRATION                          L  OUTPUT
*     LDC    - DOPING CONCENTRATION                           L  OUTPUT
*     LEL    - LATERAL FIELD                                  L  OUTPUT
*     LET    - TRANSVERSAL FIELD                              L  OUTPUT
*     LGRDFZ - FREEZE GRID                                    L  OPTION
*     LINS   - INSULATOR QUANTITIES                           L  OUTPUT
*     LJL    - LATERAL CURRENT DENSITY                        L  OUTPUT
*     LJT    - TRANSVERSAL CURRENT DENSITY                    L  OUTPUT
*     LMAJ   - MAJORITIES                                     L  OUTPUT
*     LMIN   - MINORITIES                                     L  OUTPUT
*     LMOB   - MOBILITY                                       L  OUTPUT
*     LNON   - NO DISTRIBUTIONS                               L  OUTPUT
*     LPHI   - QUASI-FERMIPOTENTIAL                           L  OUTPUT
*     LPHYCK - CHECK FOR PHYSICAL ERRORS IN THE INPUT DECK    L  OPTION
*     LPSI   - ELECTROSTATTIC POTENTIAL                       L  OUTPUT
*     LRHO   - SPACE CHARGE                                   L  OUTPUT
*     LSAVEB - FLAG TO PRODUCE BINARY FILE                    L  END
*     LTC    - CARRIER TEMPERATURE                            L  OUTPUT
*     MODEL  - DEGREE OF SOPHISTICATION                       I  OPTION
*              1 = 1-D ONE CARRIER
*              2 = 1-D ONE CARRIER THRESHOLD ITERATION
*              3 = 2-D ONE CARRIER
*              4 = AVL TWO CARRIER
*              5 = HOT TWO CARRIER WITH LOCAL ENERGY
*     NUB    - BULK INCREMENT STEPS                           I  STEP
*     NUD    - DRAIN INCREMENT STEPS                          I  STEP
*     NUG    - GATE INCREMENT STEPS                           I  STEP
*     OMEGA  - FREQUENCY FOR SMALL SIGNAL ANALYSIS            R  OPTION
*     RBULK  - EFFECTIVE BULK RESISTOR                        R  OPTION
*     RDRAIN - EFFECTIVE DRAIN RESISTOR                       R  OPTION
*     RPS    - FIXED SURFACE STATES DENSITY AT BULK           R  INTERF.
*     RSOURC - EFFECTIVE SOURCE RESISTOR                      R  OPTION
*     SDAKEV - S/D IMPLANTATION ENERGIES                      R  PROFILE
*              ARRAY OF 2 ELEMENTS
*     SDDOSE - S/D IMPLANTATION DOSES                         R  PROFILE
*              ARRAY OF 2 ELEMENTS
*     SDTOX  - S/D IMPLANTATION OXIDE THICKNESS               R  PROFILE
*     SFSS   - FAST (CHARGEABLE) SURFACE STATES DENSITY       R  INTERF.
*     SGAP   - SOURCE CONTACT GAP                             R  DEVICE
*     SIMTEM - SIMULATION TEMPERATURE                         R  OPTION
*     SISS   - STANDARD DEVIATION OF INTERFACE-STATES PROFILE R  INTERF.
*     SNSS   - SLOW (FIXED) SURFACE STATES DENSITY            R  INTERF.
*     SRV    - SURFACE RECOMBINATION VELOCITIES               R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     SUBS   - SUBSTRATE DOPING                               R  PROFILE
*     SURF   - S/D PREDEPOSITION CONCENTRATION                R  PROFILE
*     TAU    - CARRIER LIFETIMES                              R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     TEMP   - S/D PROFILE DIFFUSION TEMPERATURE              R  PROFILE
*     THCUR  - REFERENCE CURRENT FOR THRESHOLD DEFINITION     R  OPTION
*     TIME   - S/D DIFFUSION TIME                             R  PROFILE
*     TINS   - INSULATOR THICKNESS                            R  DEVICE
*     TITLE  - TITLE OF INPUT DECK                            C  1. LINE
*     UB     - APPLIED BULK VOLTAGE                           R  BIAS
*     UD     - APPLIED DRAIN VOLTAGE                          R  BIAS
*     UFB    - FLAT BAND VOLTAGE                              R  OPTION
*     UG     - APPLIED GATE VOLTAGE                           R  BIAS
*     US     - APPLIED SOURCE VOLTAGE                         R  BIAS
*     UTP    - WEIGHTS FOR ENERGY RELAXATION TIMES            R  OPTION
*              ARRAY OF 2 ELEMENTS
*     VI     - IMAGINARY PART OF SMALL SIGNAL BIAS            R  BIAS
*     VR     - REAL PART OF SMALL SIGNAL BIAS                 R  BIAS
*     VRV    - INTERFACE RECOMBINATION VELOCITIES (BULK)      R  RECOMB
*              ARRAY OF 2 ELEMENTS
*     WMB    - MERGE COEFFICIENT WEIGHT                       R  MOBIL
*     WMI    - IMPURITY SCATTERING WEIGHT                     R  MOBIL
*     WML    - LATTICE SCATTERING WEIGHT                      R  MOBIL
*     WMR    - SURFACE SCATTERING WEIGHT FOR REFERENCE MOBIL. R  MOBIL
*     WMS    - SURFACE SCATTERING WEIGHT FOR DISTANCE         R  MOBIL
*     WMT    - SURFACE SCATTERING WEIGHT FOR PREFACTOR        R  MOBIL
*     WMV    - SATURATION VELOCITY WEIGHT                     R  MOBIL
*     XISS   - PROJECTED RANGE OF INTERFACE-STATES PROFILE    R  INTERF.
*     XLGEO  - CHANNEL LENGTH                                 R  DEVICE
*     XOFF   - MASK OFFSET FOR HEAVY-DOPED S/D PROFILE        R  PROFILE
*     XWGEO  - CHANNEL WIDTH                                  R  DEVICE
*     YISUBS - INSULATOR SUBSTRATE THICKNESS                  R  DEVICE
*     YOFF   - VERTICAL OFFSET OF S/D PROFILE                 R  PROFILE
*     YSEMI  - EPITAXIAL SI-LAYER THICKNESS                   R  DEVICE
*
      INCLUDE 'CMDATX.INC'

**    INTERFACE CELLS WITHIN DIRECTIVES PROCESSOR
*
*     VAR.     DESCRIPTION                                  TYPE
*     -----------------------------------------------------------
*     BYTE   - INPUT LINE BUFFER                              C
*     ILENV  - LENGTH OF VALUE TOKEN                          I
*     INDK   - INDEX OF KEY TOKEN                             I
*     INDKO  - INDEX OF OLD KEY TOKEN                         I
*     INDV   - INDEX OF VALUE TOKEN                           I
*
*     THE FOLLOWING VARIABLES ARE OCCURENCE FLAGS FOR INPUT DATA
*
*     L1ALPH - IONIZATION PARAMETER CONSTANTS                 L
*              ARRAY OF 2 ELEMENTS
*     L1AUGE - AUGER COEFFICIENTS                             L
*              ARRAY OF 2 ELEMENTS
*     L1BETA - IONIZATION PARAMETER CONSTANTS                 L
*              ARRAY OF 2 ELEMENTS
*     L1CHAK - CHANNEL IMPLANTATION ENERGY                    L
*              ARRAY OF 2 ELEMENTS
*     L1CHDO - CHANNEL IMPLANTATION DOSES                     L
*              ARRAY OF 2 ELEMENTS
*     L1CHTE - CHANNEL IMPLANTATION DIFFUSION TEMPERATURE     L
*     L1CHTI - CHANNEL IMPLANTATION DIFFUSION TIME            L
*     L1CINT - INTRINSIC CONCENTRATION                        L
*     L1CISS - PEAK CONCENTRATION OF INTERFACE-STATES PROFILE L
*     L1DGAP - DRAIN CONTACT GAP                              L
*     L1DUB  - BULK VOLTAGE INCREMENT                         L
*     L1DUD  - DRAIN VOLTAGE INCREMENT                        L
*     L1DUG  - GATE VOLTAGE INCREMENT                         L
*     L1EINS - PERMITTIVITY OF GATE INSULATOR                 L
*     L1EISU - PERMITTIVITY OF INSULATING SUBSTRATE           L
*     L1ERRC - DESIRED ACCURACY FOR CAPACITANCES              L
*     L1ERRO - DESIRED ACCURACY FOR SPACE CHARGE              L
*     L1ERRT - DESIRED ACCURACY FOR CARRIER TEMPERATURES      L
*     L1FITL - LATERAL FIT PARAMETER FOR S/D PROFILE          L
*     L1GATE - GATE MATERIAL INDICATOR                        L
*     L1ICHA - DEVICE TYPE INDICATOR (P/N)                    L
*     L1ICHE - CHANNEL IMPLANTATION ELEMENTS                  L
*              ARRAY OF 2 ELEMENTS
*     L1IFIL - TECHNOLOGY DATA ON FILE INDEX                  L
*     L1ISDE - S/D IMPLANTATION ELEMENTS                      L
*              ARRAY OF 2 ELEMENTS
*     L1JCAP - NODE INDEX FOR SMALL SIGNAL BIAS               L
*     L1LALL - ALL DISTRIBUTIONS                              L
*     L1LASY - NON-SYMMETRIC S/D PROFILE FLAG                 L
*     L1LAVL - AVALANCHE GENERATION                           L
*     L1LCC  - CARRIER CONCENTRATION                          L
*     L1LDC  - DOPING CONCENTRATION                           L
*     L1LEL  - LATERAL FIELD                                  L
*     L1LET  - TRANSVERSAL FIELD                              L
*     L1LGRD - FREEZE GRID                                    L
*     L1LINS - INSULATOR QUANTITIES                           L
*     L1LJL  - LATERAL CURRENT DENSITY                        L
*     L1LJT  - TRANSVERSAL CURRENT DENSITY                    L
*     L1LMAJ - MAJORITIES                                     L
*     L1LMIN - MINORITIES                                     L
*     L1LMOB - MOBILITY                                       L
*     L1LNON - NO DISTRIBUTIONS                               L
*     L1LPHI - QUASI-FERMIPOTENTIAL                           L
*     L1LPHY - CHECK FOR PHYSICAL ERRORS IN THE INPUT DECK    L
*     L1LPSI - ELECTRIC POTENTIAL                             L
*     L1LRHO - SPACE CHARGE                                   L
*     L1LSAV - FLAG TO PRODUCE BINARY FILE                    L
*     L1LTC  - CARRIER POTENTIALS                             L
*     L1MODE - DEGREE OF SOPHISTICATION                       L
*     L1NUB  - BULK INCREMENT STEPS                           L
*     L1NUD  - DRAIN INCREMENT STEPS                          L
*     L1NUG  - GATE INCREMENT STEPS                           L
*     L1OMEG - FREQUENCY FOR SMALL SIGNAL ANALYSIS            L
*     L1RBUL - EFFECTIVE BULK RESISTOR                        L
*     L1RDRA - EFFECTIVE DRAIN RESISTOR                       L
*     L1RPS  - FIXED SURFACE STATES DENSITY AT BULK           L
*     L1RSOU - EFFECTIVE DRAIN RESISTOR                       L
*     L1SDAK - S/D IMPLANTATION ENERGIES                      L
*              ARRAY OF 2 ELEMENTS
*     L1SDDO - S/D IMPLANTATION DOSES                         L
*              ARRAY OF 2 ELEMENTS
*     L1SDTO - S/D IMPLANTATION OXIDE THICKNESS               L
*     L1SFSS - FAST (CHARGEABLE) SURFACE STATES DENSITY       L
*     L1SGAP - SOURCE CONTACT GAP                             L
*     L1SIMT - SIMULATION TEMPERATURE                         L
*     L1SISS - STANDARD DEVIATION OF INTERFACE-STATES PROFILE L
*     L1SNSS - SLOW (FIXED) SURFACE STATES DENSITY            L
*     L1SRV  - SURFACE RECOMBINATION VELOCITIES               L
*              ARRAY OF 2 ELEMENTS
*     L1SUBS - SUBSTRATE DOPING                               L
*     L1SURF - S/D PREDEPOSITION CONCENTRATION                L
*     L1TAU  - CARRIER LIFETIMES                              L
*              ARRAY OF 2 ELEMENTS
*     L1TEMP - S/D PROFILE DIFFUSION TEMPERATURE              L
*     L1THCU - REFERENCE CURRENT FOR THRESHOLD DEFINITION     L
*     L1TIME - S/D DIFFUSION TIME                             L
*     L1TINS - INSULATOR THICKNESS                            L
*     L1UB   - APPLIED BULK VOLTAGE                           L
*     L1UD   - APPLIED DRAIN VOLTAGE                          L
*     L1UFB  - FLAT BAND VOLTAGE                              L
*     L1UG   - APPLIED GATE VOLTAGE                           L
*     L1US   - APPLIED SOURCE VOLTAGE                         L
*     L1UTP  - WEIGHTS FOR ENERGY RELAXATION TIMES            L
*              ARRAY OF 2 ELEMENTS
*     L1VI   - IMAGINARY PART OF SMALL SIGNAL BIAS            L
*     L1VR   - REAL PART OF SMALL SIGNAL BIAS                 L
*     L1VRV  - INTERFACE RECOMBINATION VELOCITIES (BULK)      L
*              ARRAY OF 2 ELEMENTS
*     L1WMB  - MERGE COEFFICIENT WEIGHT                       L
*     L1WMI  - MOBILITY SCATTERING WEIGHT                     L
*     L1WML  - LATTICE SCATTERING WEIGHT                      L
*     L1WMR  - SURFACE SCATTERING WEIGHT FOR REFERENCE MOBIL. L
*     L1WMS  - SURFACE SCATTERING WEIGHT FOR DISTANCE         L
*     L1WMT  - SURFACE SCATTERING WEIGHT FOR PREFACTOR        L
*     L1WMV  - SATURATION VELOCITY WEIGHT                     L
*     L1XISS - PROJECTED RANGE OF INTERFACE-STATES PROFILE    L
*     L1XLGE - CHANNEL LENGTH                                 L
*     L1XOFF - MASK OFFSET FOR HEAVY-DOPED S/D PROFILE        L
*     L1XWGE - CHANNEL WIDTH                                  L
*     L1YISU - INSULATING SUBSTRATE THICKNESS                 L
*     L1YOFF - VERTICAL OFFSET OF S/D PROFILE                 L
*     L1YSEM - EPITAXIAL SI-LAYER THICKNESS                   L
*     RHIGH  - MAXIMUM ALLOWED VALUE FOR AN INPUT PARAMETER   R
*     RLOW   - MINIMUM ALLOWED VALUE FOR AN INPUT PARAMETER   R
*
      INCLUDE 'CMOS01X.INC'

**    PHYSICAL CONSTANTS AND PARAMETERS
*
*     VAR.     DESCRIPTION                                  TYPE
*     -----------------------------------------------------------
*     CAPMAT - CAPACITANCE MATRIX                             R
*              ARRAY OF 16 ELEMENTS
*     CCMAX  - MAXIMUM CARRIER CONCENTRATION                  R
*     CCMIN  - MINIMUM CARRIER CONCENTRATION                  R
*     CNOR   - NORMALIZATION FACTOR FOR CONCENTRATIONS        R
*     CSTAMA - EFFECTIVE DENSITY OF STATES OF MAJORITIES      R
*     CSTAMI - EFFECTIVE DENSITY OF STATES OF MINORITIES      R
*     DFWX   - CLOSE TO DERIVATIVES OF FWX                    R
*              ARRAY OF (MAXNPT-1) ELEMENTS
*     DFWY   - CLOSE TO DERIVATIVES OF FWY                    R
*              ARRAY OF (MAXNPT-1) ELEMENTS
*     DPHMAX - LOGARITHM OF MAXIMUM CARRIER CONCENTRATION     R
*     DPHMIN - LOGARITHM OF MINIMUM CARRIER CONCENTRATION     R
*     DX     - MESH INCREMENTS IN X-DIRECTION                 R
*              ARRAY OF (MAXNPT-1) ELEMENTS
*     DXM    - AVERAGED MESH INCREMENTS IN X-DIRECTION        R
*              ARRAY OF MAXNPT ELEMENTS
*     DY     - MESH INCREMENTS IN Y-DIRECTION                 R
*              ARRAY OF (MAXNPT-1) ELEMENTS
*     DYM    - AVERAGED MESH INCREMENTS IN Y-DIRECTION        R
*              ARRAY OF MAXNPT ELEMENTS
*     EGAP   - ENERGY GAP OF FORBIDDEN BAND                   R
*     ESEM   - PERMITTIVITY CONSTANT OF SILICON               R
*     FLUNOR - NORMALIZATION FACTOR FOR TERMINAL CURRENTS     R
*     FLUX1D - DRAIN CURRENT OF MODEL=1-D                     R
*     FLUX2D - DRAIN CURRENT OF MODEL=2-D                     R
*     FLUXAV - BULK AND DRAIN CURRENT OF MODEL=AV             R
*              ARRAY OF 2 ELEMENTS
*     FLUXB  - BULK CURRENT COMPONENTS                        R
*              ARRAY OF 2 ELEMENTS
*     FLUXD  - DRAIN CURRENT COMPONENTS                       R
*              ARRAY OF 2 ELEMENTS
*     FWX    - X-WEIGHTS FOR TERMINAL CURRENT INTEGRATION     R
*              ARRAY OF MAXNPT ELEMENTS
*     FWY    - Y-WEIGHTS FOR TERMINAL CURRENT INTEGRATION     R
*              ARRAY OF MAXNPT ELEMENTS
*     IFAIL  - FAILURE PARAMETER                              I
*              0 = NO FAILURE
*              1 = DOPING PROFILE INCONSISTENT
*              2 = DRAIN CURRENT ITERATION NOT CONVERGENT
*              3 = THRESHOLD ITERATION NOT CONVERGENT
*              4 = HOT ITERATION NOT CONVERGENT
*     IMODEL - ACTIVE MODEL                                   I
*     IPM    - ACTUAL MEMORY DEPTH FOR ACCELERATION           I
*     IPMPT  - MEMORY POINTER FOR ACCELERATION BUFFER         I
*     ITCYC  - ITERATION CYCLE COUNTER                        I
*     ITGRD  - COUNT OF GRID ADAPTIONS                        I
*     ITHOT  - HOT ITERATION CYCLE COUNTER                    I
*     LBOTI  - FLAG FOR INSULATING SUBSTRATE                  L
*     MAXDO1 - MAXIMUM NUMBER OF POINTS FOR 1-DIMENSIONAL     I PAR
*              DOPING INPUT FILE
*     MAXDO2 - MAXIMUM NUMBER OF MESHLINES IN X- AND Y-DIR.   I PAR
*              FOR 2-DIMENSIONAL DOPING INPUT FILE
*     MAXIPM - MAXIMUM MEMORY DEPTH FOR ACCELERATION          I PAR
*     MAXNPT - MAXIMUM NUMBER OF MESHLINES IN X- AND Y-DIR.   I PAR
*     NT     - NUMBER OF ACTIVE POINTS IN Y-DIRECTION         I
*     NX     - NUMBER OF POINTS IN X-DIRECTION                I
*     NXD    - INDEX OF START OF DRAIN CONTACT                I
*     NXG1   - INDEX OF START OF GATE CONTACT                 I
*     NXG2   - INDEX OF END OF GATE CONTACT                   I
*     NXJD   - INDEX OF SUBDIFFUSION AT DRAIN                 I
*     NXJS   - INDEX OF SUBDIFFUSION AT SOURCE                I
*     NXS    - INDEX OF END OF SOURCE CONTACT                 I
*     NXT    - = NX * NT                                      I
*     NXTMX  - = NXT - NX                                     I
*     NXY    - = NX * NY                                      I
*     NXYB   - NUMBER OF ACTIVE POINTS IN THE SEMICONDUCTOR   I
*     NXYMX  - = NXY - NX                                     I
*     NXYOX  - = NX * ( NYOX - 1 )                            I
*     NXYSEM - = NX * NYSEM                                   I
*     NXYTOT - = NX * NYTOT                                   I
*     NY     - NUMBER OF POINTS IN Y-DIRECTION (SEMICONDUCTOR)I
*     NYG    - INDEX OF START OF GATE CONTACT (Y-DIRECTION)   I
*     NYJD   - INDEX OF JUNCTION DEPTH AT DRAIN               I
*     NYJS   - INDEX OF JUNCTION DEPTH AT SOURCE              I
*     NYOX   - INDEX OF GATE INTERFACE                        I
*     NYSEM  - NUMBER OF POINTS IN Y-DIRECTION (TOP INS.+SEM) I
*     NYTOT  - NUMBER OF POINTS IN Y-DIRECTION (TOTAL)        I
*     ONE    - = 1.                                           R PAR
*     PMP    - ACCELERATION COEFFICIENTS                      R
*              ARRAY OF ((MAXIPM+1)*MAXIPM) ELEMENTS
*     PNOR2  - ACHIEVED L-2 ERROR                             R
*     PNORH  - ACHIEVED ERROR OF CARRIER TEMPERATURE          R
*     PNORM  - ACHIEVED L-INF ERROR                           R
*     PNORMD - ACHIEVED L-INF ERROR OF INCREMENTS             R
*              ARRAY OF 3 ELEMENTS
*     RMAEFF - RELATIVE EFFECTIVE MASS OF MAJORITIES          R
*     RMIEFF - RELATIVE EFFECTIVE MASS OF MINORITIES          R
*     Q      - ELEMENTARY CHARGE                              R PAR
*     SECAUR - SECTIO AUREA CONSTANT = (1+SQRT(5))/2          R
*     TAUEN  - ENERGY RELXATION TIMES                         R
*              ARRAY OF 2 ELEMENTS
*     UF     - FERMI-VOLTAGE                                  R
*     UGS    - EFFECTIVE GATE-SOURCE VOLTAGE                  R
*     UIDN   - NEW DRAIN BIAS INCREMENT DUE TO CURRENT        R
*     UIDO   - OLD DRAIN BIAS INCREMENT DUE TO CURRENT        R
*     UISN   - NEW SOURCE BIAS INCREMENT DUE TO CURRENT       R
*     UISO   - OLD SOURCE BIAS INCREMENT DUE TO CURRENT       R
*     UT     - THERMAL VOLTAGE                                R
*     VSAT   - SATURATION VELOCITIES                          R
*              ARRAY OF 2 ELEMENTS
*     X      - MESH COORDINATES IN X-DIRECTION                R
*              ARRAY OF MAXNPT ELEMENTS
*     XJD    - SUBDIFFUSION AT DRAIN                          R
*     XJS    - SUBDIFFUSION AT SOURCE                         R
*     XLPRO1 - DRAIN PROFILE EDGE FOR FIRST PROFILE           R
*     XLPRO2 - DRAIN PROFILE EDGE FOR SECOND PROFILE          R
*     XYNOR  - NORMALIZATION FACTOR FOR LENGTHS               R
*     Y      - MESH COORDINATES IN Y-DIRECTION                R
*              ARRAY OF MAXNPT ELEMENTS
*     YJD    - JUNCTION DEPTH AT DRAIN                        R
*     YJS    - JUNCTION DEPTH AT SOURCE                       R
*     ZERO   - = 0.                                           R PAR
*     ZMOB   - LATTICE MOBILITIES                             R
*              ARRAY OF 2 ELEMENTS
*
      INCLUDE 'CMOS02X.INC'

**    MEMORY MANAGEMENT PARAMETERS
*
*     VAR.     DESCRIPTION                                  TYPE
*     -----------------------------------------------------------
*     IBEB   - SLOT INDICES FOR BERNOULLI WEIGHTS             I
*              ARRAY OF 4 ELEMENTS
*              (1) SUB DIAGONAL WEIGHT FOR MINORITIES
*              (2) SUB DIAGONAL WEIGHT FOR MAJORITIES
*              (3) SUPER-SUB DIAGONAL WEIGHT FOR MINORITIES
*              (4) SUPER-SUB DIAGONAL WEIGHT FOR MAJORITIES
*     IBEP   - SLOT INDICES FOR BERNOULLI WEIGHTS             I
*              ARRAY OF 4 ELEMENTS
*              (1) SUP DIAGONAL WEIGHT FOR MINORITIES
*              (2) SUP DIAGONAL WEIGHT FOR MAJORITIES
*              (3) SUPER-SUP DIAGONAL WEIGHT FOR MINORITIES
*              (4) SUPER-SUP DIAGONAL WEIGHT FOR MAJORITIES
*     ICACC  - SLOT INDEX FOR ACCEPTOR CONCENTRATION          I
*     ICC    - SLOT INDICES FOR CARRIER CONCENTRATIONS        I
*              ARRAY OF 4 ELEMENTS
*              (1) FOR MINORITY CARRIER CONCENTRATION
*              (2) FOR MAJORITY CARRIER CONCENTRATION
*              (3) FOR MINORITY CARRIER CONCENTRATION BACKUP
*              (4) FOR MAJORITY CARRIER CONCENTRATION BACKUP
*     ICDON  - SLOT INDEX FOR DONOR CONCENTRATION             I
*     IDIF   - SLOT INDICES FOR CARRIER DIFFUSIITIES          I
*              ARRAY OF 4 ELEMENTS
*              (1) FOR LATERAL MINORITY CARRIER MOBILITY
*              (2) FOR LATERAL MAJORITY CARRIER MOBILITY
*              (3) FOR TRANSVERSAL MINORITY CARRIER MOBILITY
*              (4) FOR TRANSVERSAL MAJORITY CARRIER MOBILITY
*     IDSTK  - SLOT INDICES FOR ACCELERATION                  I
*              ARRAY OF (MAXIPM) ELEMENTS
*     IGEN   - SLOT INDEX FOR IMPACT IONIZATION RATE          I
*     IJD    - SLOT INDICES FOR CURRENT DENSITIES             I
*              ARRAY OF 4 ELEMENTS
*              (1) FOR LATERAL MINORITY CURRENT DENSITY
*              (2) FOR LATERAL MAJORITY CURRENT DENSITY
*              (3) FOR TRANSVERSAL MINORITY CURRENT DENSITY
*              (4) FOR TRANSVERSAL MAJORITY CURRENT DENSITY
*     IPOI   - SLOT INDICES FOR POISSON COEFFICIENTS          I
*              ARRAY OF 4 ELEMENTS
*              (1) FOR SUB/SUP DIAGONAL COEFFICIENT
*              (2) FOR SUPER-SUB/SUP DIAGONAL COEFFICIENT
*              (3) FOR MAIN DIAGONAL COEFFICIENT
*              (4) FOR DISCRETIZATION AREAS
*     IPSI   - SLOT INDEX FOR ELECTROSTATIC POTENTIAL         I
*     ITC    - SLOT INDICES FOR ELECTRONIC VOLTAGES           I
*              ARRAY OF 2 ELEMENTS
*              (1) FOR MINORITY CARRIER ELECTRONIC VOLTAGE
*              (2) FOR MAJORITY CARRIER ELECTRONIC VOLTAGE
*     IYSTK  - SLOT INDICES FOR SOLUTION MEMORY               I
*              ARRAY OF (MAXIPM) ELEMENTS
*     JP     - START ADDRESSES OF ARRAYS IN PAD               I
*              ARRAY OF (MAXPAD) ELEMENTS
*     JSLPRE - PRESET VALUE FOR UNDEFINED SLOTS               I PAR
*     MAXPAD - MAXIMUM NUMBER OF SLOTS IN WORKING BUFFER      I PAR
*     MAXSLT - MAXIMUM NUMBER OF SLOTS ON SLOT FILE           I PAR
*
      INCLUDE 'CMEMX.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
**    PRESET COMMON BLOCKS
      CALL PDATA
**    PUT START TIME INTO CPUTIM
      CALL ZZZSEC (CPUTIM)
**    EXECUTE INPUT PROCESSOR
      CALL SUB01
**    EXECUTE NUMERICS PROCESSOR
      CALL SUB02
**    STOP HERE AFTER HAVING DONE EVERYTHING
      STOP '*MINIMOS* DONE'

      END
      SUBROUTINE ABTMOS (IABTPT)
      INCLUDE 'TYPES.INC'

***   ABTMOS - *MINIMOS* ABORT PROCESSOR
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      WRITE (NUMOUT,'(''0'')')

      GOTO (1,2,3,4,5,6), IABTPT
**    FALL THROUGH COMP.GOTO INDICATES INTERNAL ERROR
      WRITE (NUMOUT,8) 'FATAL INTERNAL ERROR'
      WRITE (NUMREF,8) 'FATAL INTERNAL ERROR'
      STOP 'FATAL INTERNAL ERROR'

 1    IF (NERSYN.NE.0) THEN
         WRITE (NUMOUT,7) NERSYN,'SYNTAX ERROR(S)'
         WRITE (NUMREF,7) NERSYN,'SYNTAX ERROR(S)'
      ENDIF
      IF (NEROCC.NE.0) THEN
         WRITE (NUMOUT,7) NEROCC,'OCCURENCE ERROR(S)'
         WRITE (NUMREF,7) NEROCC,'OCCURENCE ERROR(S)'
      ENDIF

 2    IF (NERPHY.NE.0) THEN
         WRITE (NUMOUT,7) NERPHY,'PHYSICAL ERROR(S)'
         WRITE (NUMREF,7) NERPHY,'PHYSICAL ERROR(S)'
      ENDIF
      STOP 'ERROR(S) ON INPUT FILE'

 3    WRITE (NUMOUT,8) 'INPUT FILE EMPTY'
      WRITE (NUMREF,8) 'INPUT FILE EMPTY'
      STOP 'INPUT FILE EMPTY'

 4    WRITE (NUMOUT,8) 'DOPING FILE EMPTY'
      WRITE (NUMREF,8) 'DOPING FILE EMPTY'
      STOP 'DOPING FILE EMPTY'

 5    WRITE (NUMOUT,8) 'INSUFFICIENT DATA ON DOPING FILE'
      WRITE (NUMREF,8) 'INSUFFICIENT DATA ON DOPING FILE'
      STOP 'INSUFFICIENT DATA ON DOPING FILE'

 6    WRITE (NUMOUT,8) 'DOPING FILE FORMAT ERROR'
      WRITE (NUMREF,8) 'DOPING FILE FORMAT ERROR'
      STOP 'DOPING FILE FORMAT ERROR'

 7    FORMAT (T2,I8,T11,A)
 8    FORMAT (T12,A)
      END
      SUBROUTINE WRTHED
      INCLUDE 'TYPES.INC'

***   WRTHED - WRITE A HEADER
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'

**    INCREMENT PAGE COUNTER
      NPAGE=NPAGE+1
**    WRITE HEADING
      WRITE (NUMOUT,'(''1''/T11,''MINIMOS 4.0 PC   '',4X,''TU VIENNA'',
     111X,A,2X,''PAGE'',I4/T11,A/''0'')') DATTIM,NPAGE,TITLE

      END
      SUBROUTINE ZZZDAT (DATTIM)

***   ZZZDAT - GET DATE AND TIME
*     (PROFORT/RMFORT VERSION)
*
*     S.SELBERHERR  1985/12/xx
*
      INTEGER*2 IYE,IMO,IDA,IHO,IMI,ISE,IHS
      CHARACTER*20 DATTIM

      CALL GETTIM (IHO,IMI,ISE,IHS)
      CALL GETDAT (IYE,IMO,IDA)
      WRITE (DATTIM,1) MOD(IYE,100),IMO,IDA,IHO,IMI,ISE
 1    FORMAT (1X,I2.2,2('/',I2.2),2X,I2.2,2('.',I2.2),1X)

      END
      SUBROUTINE ZZZSEC (CPUTIM)
      INCLUDE 'TYPES.INC'

***   ZZZSEC - GET CPU-TIME SINCE START OF JOB
*     (PROFORT/RMFORT VERSION)
*
*     S.SELBERHERR  1985/12/xx
*
      INTEGER*2 IHO,IMI,ISE,IHS

      CALL GETTIM (IHO,IMI,ISE,IHS)
      CPUTIM=0.01*IHS+ISE+60*(IMI+60*IHO)

      END
